@@ -22,8 +22,9 @@
 static const char s_schemePropertyName[] = "KDE_COLOR_SCHEME_PATH";
 static const QByteArray s_blurBehindPropertyName = QByteArrayLiteral("ENABLE_BLUR_BEHIND_HINT");
 
-X11Integration::X11Integration()
+X11Integration::X11Integration(KdePlatformTheme *platformTheme)
     : QObject()
+    , m_platformTheme(platformTheme)
 {
 }
 
@@ -50,6 +51,7 @@ bool X11Integration::eventFilter(QObject *watched, QEvent *event)
             QPlatformSurfaceEvent *pe = static_cast<QPlatformSurfaceEvent *>(event);
             if (!w->flags().testFlag(Qt::ForeignWindow)) {
                 if (pe->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated) {
+                    m_platformTheme->windowCreated(w);
                     auto flags = w->flags();
                     // A recent KWin change means it now follows WindowButtonHints on X11
                     // Some KDE applications use QDialogs for their main window,
@@ -57,17 +59,11 @@ bool X11Integration::eventFilter(QObject *watched, QEvent *event)
                     // To avoid clients changing, we adjust flags here.
                     // This is documented as being only available on some platforms,
                     // so altering is relatively safe.
-                    if (flags.testFlag(Qt::Dialog)) {
+                    if (flags.testFlag(Qt::Dialog) && !flags.testFlag(Qt::CustomizeWindowHint)) {
                         if (!w->transientParent()) {
-                            flags.setFlag(Qt::WindowMinimizeButtonHint, true);
+                            flags.setFlag(Qt::WindowCloseButtonHint);
+                            flags.setFlag(Qt::WindowMinMaxButtonsHint);
                         }
-
-                        // QWINDOWSIZE_MAX from qwindow_p.h
-                        const auto maxWindowSize = ((1 << 24) - 1);
-                        if (w->maximumSize() == QSize(maxWindowSize, maxWindowSize)) {
-                            flags.setFlag(Qt::WindowMaximizeButtonHint, true);
-                        }
-
                         w->setFlags(flags);
                     }
 
